@@ -1,9 +1,10 @@
 import json
 from distutils.version import LooseVersion
 
+import numpy
 import pytest
 
-from pyproj import CRS, Transformer, proj_version_str
+from pyproj import CRS, __proj_version__
 from pyproj.crs import (
     CoordinateOperation,
     CoordinateSystem,
@@ -14,6 +15,7 @@ from pyproj.crs import (
 from pyproj.crs.enums import CoordinateOperationType, DatumType
 from pyproj.enums import ProjVersion, WktVersion
 from pyproj.exceptions import CRSError
+from pyproj.transformer import TransformerGroup
 
 
 class CustomCRS(object):
@@ -77,6 +79,12 @@ def test_from_string():
         )
 
 
+def test_initialize_projparams_with_kwargs():
+    crs_mixed_args = CRS("+proj=utm +zone=10", ellps="WGS84")
+    crs_positional = CRS("+proj=utm +zone=10 +ellps=WGS84")
+    assert crs_mixed_args.is_exact_same(crs_positional)
+
+
 def test_bare_parameters():
     """ Make sure that bare parameters (e.g., no_defs) are handled properly,
     even if they come in with key=True.  This covers interaction with pyproj,
@@ -277,6 +285,14 @@ def test_repr_compound():
     )
 
 
+def test_axis_info_compound():
+    assert [axis.direction for axis in CRS.from_epsg(3901).axis_info] == [
+        "north",
+        "east",
+        "up",
+    ]
+
+
 def test_dunder_str():
     with pytest.warns(FutureWarning):
         assert str(CRS({"init": "EPSG:4326"})) == CRS({"init": "EPSG:4326"}).srs
@@ -490,7 +506,8 @@ def test_coordinate_operation():
     crs = CRS.from_epsg(26915)
     assert repr(crs.coordinate_operation) == (
         "<Coordinate Operation: Conversion>\n"
-        "UTM zone 15N\n"
+        "Name: UTM zone 15N\n"
+        "Method: Transverse Mercator\n"
         "Area of Use:\n"
         "- name: World - N hemisphere - 96°W to 90°W\n"
         "- bounds: (-96.0, 0.0, -90.0, 84.0)"
@@ -522,11 +539,18 @@ def test_coordinate_operation():
 
 def test_coordinate_operation_grids():
     cc = CoordinateOperation.from_epsg(1312)
-    assert (
-        repr(cc.grids)
-        == "[Grid(short_name=NTv1_0.gsb, full_name=, package_name=, url=, "
-        "direct_download=False, open_license=False, available=False)]"
-    )
+    if not cc.grids[0].full_name:
+        assert (
+            repr(cc.grids)
+            == "[Grid(short_name=NTv1_0.gsb, full_name=, package_name=, url=, "
+            "direct_download=False, open_license=False, available=False)]"
+        )
+    else:
+        assert (
+            repr(cc.grids)
+            == "[Grid(short_name=NTv1_0.gsb, full_name=NTv1_0.gsb, package_name=, "
+            "url=, direct_download=False, open_license=False, available=False)]"
+        )
 
 
 def test_coordinate_operation_grids__alternative_grid_name():
@@ -536,9 +560,11 @@ def test_coordinate_operation_grids__alternative_grid_name():
     assert grid.direct_download is True
     assert grid.open_license is True
     assert grid.available is True
-    if LooseVersion(proj_version_str) >= LooseVersion("7.0.0"):
+    if LooseVersion(__proj_version__) >= LooseVersion("7.0.0"):
         assert grid.short_name == "ca_nrc_ntv1_can.tif"
-        assert grid.full_name.endswith("ntv1_can.dat")
+        assert grid.full_name.endswith("ntv1_can.dat") or grid.full_name.endswith(
+            grid.short_name
+        )
         assert grid.package_name == ""
         assert grid.url == "https://cdn.proj.org/ca_nrc_ntv1_can.tif"
     else:
@@ -793,6 +819,14 @@ def test_coordinate_operation_towgs84_seven():
     ]
 
 
+def test_axis_info_bound():
+    crs = CRS(
+        "+proj=tmerc +lat_0=0 +lon_0=15 +k=0.9996 +x_0=2520000 +y_0=0 "
+        "+ellps=intl +towgs84=-122.74,-34.27,-22.83,-1.884,-3.400,-3.030,-15.62"
+    )
+    assert [axis.direction for axis in crs.axis_info] == ["east", "north"]
+
+
 def test_coordinate_operation_towgs84_missing():
     crs = CRS("epsg:3004")
     assert crs.coordinate_operation.towgs84 == []
@@ -1080,6 +1114,13 @@ def test_srs__no_plus():
 
 def test_equals_different_type():
     assert CRS("epsg:4326") != ""
+    assert not CRS("epsg:4326") == ""
+
+    assert CRS("epsg:4326") != 27700
+    assert not CRS("epsg:4326") == 27700
+
+    assert not CRS("epsg:4326") != 4326
+    assert CRS("epsg:4326") == 4326
 
 
 def test_is_exact_same_different_type():
@@ -1163,7 +1204,7 @@ def test_to_dict_no_proj4():
             "proj": "ob_tran",
         }
     )
-    if LooseVersion(proj_version_str) >= LooseVersion("6.3.0"):
+    if LooseVersion(__proj_version__) >= LooseVersion("6.3.0"):
         with pytest.warns(UserWarning):
             assert crs.to_dict() == {
                 "R": 6371229,
@@ -1214,13 +1255,14 @@ def test_operations_missing():
 
 
 def test_operations():
-    transformer = Transformer.from_crs(28356, 7856)
+    transformer = TransformerGroup(28356, 7856).transformers[0]
     coord_op = CoordinateOperation.from_string(transformer.to_wkt())
     assert coord_op.operations == transformer.operations
 
 
 def test_operations__scope_remarks():
-    transformer = Transformer.from_crs(28356, 7856)
+
+    transformer = TransformerGroup(28356, 7856).transformers[0]
     coord_op = CoordinateOperation.from_string(transformer.to_wkt())
     assert coord_op.operations == transformer.operations
     # scope does not transfer for some reason
@@ -1239,3 +1281,39 @@ def test_crs_equals():
 def test_crs_equals__ignore_axis_order():
     with pytest.warns(FutureWarning):
         assert CRS("epsg:4326").equals("+init=epsg:4326", ignore_axis_order=True)
+
+
+@pytest.mark.parametrize(
+    "crs_input",
+    [
+        "+proj=utm +zone=15",
+        26915,
+        "+proj=utm +zone=15 +towgs84=0,0,0",
+        "EPSG:26915+5717",
+    ],
+)
+def test_utm_zone(crs_input):
+    assert CRS(crs_input).utm_zone == "15N"
+
+
+@pytest.mark.parametrize("crs_input", ["+proj=tmerc", "epsg:4326"])
+def test_utm_zone__none(crs_input):
+    assert CRS(crs_input).utm_zone is None
+
+
+def test_numpy_bool_kwarg_false():
+    # Issue 564
+    south = numpy.array(50) < 0
+    crs = CRS(
+        proj="utm", zone=32, ellipsis="WGS84", datum="WGS84", units="m", south=south
+    )
+    assert "south" not in crs.srs
+
+
+def test_numpy_bool_kwarg_true():
+    # Issue 564
+    south = numpy.array(50) > 0
+    crs = CRS(
+        proj="utm", zone=32, ellipsis="WGS84", datum="WGS84", units="m", south=south
+    )
+    assert "+south " in crs.srs
