@@ -3,7 +3,7 @@ from distutils.version import LooseVersion
 import pytest
 from numpy.testing import assert_almost_equal
 
-from pyproj import CRS, proj_version_str
+from pyproj import CRS, __proj_version__
 from pyproj.crs import ProjectedCRS
 from pyproj.crs._cf1x8 import _try_list_if_string
 from pyproj.crs.coordinate_operation import (
@@ -73,6 +73,8 @@ def test_to_cf_transverse_mercator():
         "false_easting": 2520000.0,
         "false_northing": 0.0,
         "scale_factor_at_central_meridian": 0.9996,
+        "geographic_crs_name": "unknown",
+        "projected_crs_name": "unknown",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("BOUNDCRS[")
@@ -129,6 +131,9 @@ def test_from_cf_transverse_mercator(towgs84_test):
         "false_easting": 2520000.0,
         "false_northing": 0.0,
         "scale_factor_at_central_meridian": 1.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
+        "horizontal_datum_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("BOUNDCRS[")
@@ -154,6 +159,8 @@ def test_cf_from_latlon():
         "longitude_of_prime_meridian": 0.0,
         "prime_meridian_name": "Greenwich",
         "grid_mapping_name": "latitude_longitude",
+        "geographic_crs_name": "undefined",
+        "reference_ellipsoid_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("GEOGCRS[")
@@ -253,6 +260,7 @@ def test_cf_rotated_latlon():
         "grid_north_pole_latitude": 32.5,
         "grid_north_pole_longitude": 170.0,
         "north_pole_grid_longitude": 0.0,
+        "geographic_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("GEOGCRS[")
@@ -261,7 +269,7 @@ def test_cf_rotated_latlon():
     _test_roundtrip(expected_cf, "GEOGCRS[")
     with pytest.warns(UserWarning):
         proj_dict = crs.to_dict()
-    if LooseVersion(proj_version_str) >= LooseVersion("6.3.0"):
+    if LooseVersion(__proj_version__) >= LooseVersion("6.3.0"):
         assert proj_dict == {
             "proj": "ob_tran",
             "o_proj": "longlat",
@@ -287,7 +295,7 @@ def test_cf_rotated_latlon__grid():
     )
     with pytest.warns(UserWarning):
         proj_dict = crs.to_dict()
-    if LooseVersion(proj_version_str) >= LooseVersion("6.3.0"):
+    if LooseVersion(__proj_version__) >= LooseVersion("6.3.0"):
         assert proj_dict == {
             "proj": "ob_tran",
             "o_proj": "longlat",
@@ -324,6 +332,8 @@ def test_cf_lambert_conformal_conic_1sp():
         "false_easting": 0.0,
         "false_northing": 0.0,
         "standard_parallel": 25.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -372,6 +382,8 @@ def test_cf_lambert_conformal_conic_2sp(standard_parallel):
         "longitude_of_central_meridian": 265.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -422,6 +434,9 @@ def test_oblique_mercator():
         "scale_factor_at_projection_origin": 1.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
+        "horizontal_datum_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -494,6 +509,8 @@ def test_geos_crs_sweep():
         "longitude_of_projection_origin": 0.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -525,6 +542,8 @@ def test_geos_crs_fixed_angle_axis():
         "longitude_of_projection_origin": 0.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -548,6 +567,10 @@ def test_geos_proj_string():
         "longitude_of_projection_origin": 0.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "unknown",
+        "horizontal_datum_name": "unknown",
+        "projected_crs_name": "unknown",
+        "reference_ellipsoid_name": "unknown",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -587,6 +610,8 @@ def test_mercator_b():
         "longitude_of_projection_origin": 10.0,
         "false_easting": 0.0,
         "false_northing": 0.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     with pytest.warns(UserWarning):
         assert crs.to_dict() == {
@@ -664,7 +689,7 @@ def test_export_compound_crs():
 
 
 @pytest.mark.skipif(
-    LooseVersion(proj_version_str) < LooseVersion("6.3.0"),
+    LooseVersion(__proj_version__) < LooseVersion("6.3.0"),
     reason="geoid model does not work in PROJ < 6.3.0",
 )
 def test_geoid_model_name():
@@ -811,6 +836,8 @@ def test_lambert_azimuthal_equal_area():
         "longitude_of_projection_origin": 2.0,
         "false_easting": 3.0,
         "false_northing": 4.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -834,6 +861,8 @@ def test_lambert_cylindrical_equal_area():
         "longitude_of_central_meridian": 2.0,
         "false_easting": 3.0,
         "false_northing": 4.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -858,6 +887,8 @@ def test_mercator_a():
         "false_easting": 3.0,
         "false_northing": 4.0,
         "scale_factor_at_projection_origin": 1.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -881,6 +912,8 @@ def test_orthographic():
         "longitude_of_projection_origin": 2.0,
         "false_easting": 3.0,
         "false_northing": 4.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -905,6 +938,8 @@ def test_polar_stereographic_a():
         "false_easting": 2.0,
         "false_northing": 3.0,
         "scale_factor_at_projection_origin": 1.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -928,6 +963,8 @@ def test_polar_stereographic_b():
         "straight_vertical_longitude_from_pole": 1.0,
         "false_easting": 2.0,
         "false_northing": 3.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -952,6 +989,8 @@ def test_stereographic():
         "false_easting": 2.0,
         "false_northing": 3.0,
         "scale_factor_at_projection_origin": 1.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -974,6 +1013,8 @@ def test_sinusoidal():
         "longitude_of_projection_origin": 0.0,
         "false_easting": 1.0,
         "false_northing": 2.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
@@ -998,9 +1039,71 @@ def test_vertical_perspective():
         "longitude_of_projection_origin": 1.0,
         "false_easting": 2.0,
         "false_northing": 3.0,
+        "geographic_crs_name": "undefined",
+        "projected_crs_name": "undefined",
     }
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
     assert cf_dict == expected_cf
     # test roundtrip
     _test_roundtrip(expected_cf, "PROJCRS[")
+
+
+def test_build_custom_datum():
+    cf_dict = {
+        "semi_major_axis": 6370997.0,
+        "semi_minor_axis": 6370997.0,
+        "inverse_flattening": 0.0,
+        "reference_ellipsoid_name": "Normal Sphere (r=6370997)",
+        "longitude_of_prime_meridian": 1.0,
+        "grid_mapping_name": "oblique_mercator",
+        "latitude_of_projection_origin": 0.0,
+        "longitude_of_projection_origin": 13.809602948622212,
+        "azimuth_of_central_line": 8.998112717187938,
+        "scale_factor_at_projection_origin": 1.0,
+        "false_easting": 0.0,
+        "false_northing": 0.0,
+    }
+    crs = CRS.from_cf(cf_dict)
+    assert crs.datum.name == "undefined"
+    assert crs.ellipsoid.name == "Normal Sphere (r=6370997)"
+    assert crs.prime_meridian.name == "undefined"
+    assert crs.prime_meridian.longitude == 1
+
+
+def test_build_custom_datum__default_prime_meridian():
+    cf_dict = {
+        "semi_major_axis": 6370997.0,
+        "semi_minor_axis": 6370997.0,
+        "inverse_flattening": 0.0,
+        "grid_mapping_name": "oblique_mercator",
+        "latitude_of_projection_origin": 0.0,
+        "longitude_of_projection_origin": 13.809602948622212,
+        "azimuth_of_central_line": 8.998112717187938,
+        "scale_factor_at_projection_origin": 1.0,
+        "false_easting": 0.0,
+        "false_northing": 0.0,
+    }
+    crs = CRS.from_cf(cf_dict)
+    assert crs.datum.name == "undefined"
+    assert crs.ellipsoid.name == "undefined"
+    assert crs.prime_meridian.name == "Greenwich"
+    assert crs.prime_meridian.longitude == 0
+
+
+def test_build_custom_datum__default_ellipsoid():
+    cf_dict = {
+        "prime_meridian_name": "Paris",
+        "grid_mapping_name": "oblique_mercator",
+        "latitude_of_projection_origin": 0.0,
+        "longitude_of_projection_origin": 13.809602948622212,
+        "azimuth_of_central_line": 8.998112717187938,
+        "scale_factor_at_projection_origin": 1.0,
+        "false_easting": 0.0,
+        "false_northing": 0.0,
+    }
+    crs = CRS.from_cf(cf_dict)
+    assert crs.datum.name == "undefined"
+    assert crs.ellipsoid.name == "WGS 84"
+    assert crs.prime_meridian.name == "Paris"
+    assert str(crs.prime_meridian.longitude).startswith("2.")
